#include "console.h"

#include <stdio.h>
#include <sys/ioctl.h>
#include <unistd.h>

namespace tuxli {

bool Console::is_kb_hit() {
  if (hit_eof_) {
    return false;
  }
  int byteswaiting;
  ioctl(0, FIONREAD, &byteswaiting);
  if (!byteswaiting && ::write(fileno(stdin), 0, 0) != 0) {
    hit_eof_ = true;
    return false;
  }
  return byteswaiting != 0;
}

std::optional<char> Console::read_kb() {
  if (!is_kb_hit()) {
    return std::nullopt;
  }
  char got = 0;
  if (::read(fileno(stdin), &got, sizeof(got)) == 1) {
    return got;
  }
  return std::nullopt;
}

reg_t Console::write(addr_t offset, reg_t value) {
  switch (offset) {
    case 0:
      printf("%c", value);
      fflush(stdout);
      break;
  }
  return 0;
}

reg_t Console::read(addr_t offset) {
  switch (offset) {
    case 0:
      if (std::optional<char> kb = read_kb(); kb.has_value()) {
        return *kb;
      }
      return 0;

    case 5:
      return 0x60 | is_kb_hit();
  }
  return 0;
}

}  // namespace tuxli