#include <cstddef>
#include <cstdint>
#include <memory>
#include <optional>
#include <utility>
#include <vector>

#include "peripheral.h"
#include "third_party/abseil-cpp/absl/container/flat_hash_map.h"
#include "third_party/abseil-cpp/absl/types/span.h"
#include "types.h"

#pragma once

struct MiniRV32IMAState;

namespace tuxli {

// Tuxli is a RV32 core emulator.
class Tuxli {
 public:
  // Creates a new core emulator against the given RAM. `dtb_offset` is the
  // offset of the Devicetree blob in `ram`.
  Tuxli(
      absl::Span<std::byte> ram, addr_t dtb_offset,
      std::vector<std::pair<addr_t, std::unique_ptr<Peripheral>>> peripherals);
  ~Tuxli();

  // Runs the core for a short period of time. Returns `false` if the core has
  // shutdown or failed.
  bool Run();

  // Interface used by `MiniRV32IMA`.
  reg_t ram_size() const;

  reg_t csrw(reg_t csr, reg_t value);
  reg_t csrr(reg_t csr);

  reg_t other_csrw(reg_t csr, reg_t value);
  reg_t other_csrr(reg_t csr);

  reg_t exception(reg_t pc, reg_t ir, reg_t code);

 private:
  absl::Span<std::byte> ram_;
  absl::flat_hash_map<addr_t, std::unique_ptr<Peripheral>> peripherals_;
  std::unique_ptr<MiniRV32IMAState> state_;
};

}  // namespace tuxli
